#!/usr/bin/env python

import time
import datetime
import dateutil.parser
import functools


class Vehicle(object):
    """
    The Vehicle object can be used to easily access data from the tronity api.

    Parameters:
        str:vehicle_id Id of the vehicle
        Client:client Client to access the Tronity API
    """

    def __init__(self, vehicle_id, client) -> None:
        """
        The vehicle object can be used to easily access data related to the vehicle.

        :param vehicle_id: the id of the vehicle the object should represent
        :type vehicle_id:str
        :param client: client object
        :type client:Client
        """
        self.id = vehicle_id
        self._client = client
        self.data = self._get_vehicle()
        self.vin = self._get("vin")
        self.get_odometer = functools.partial(self._get, "odometer")
        self.get_battery = functools.partial(self._get, "battery")
        self.get_location = functools.partial(self._get, "location")

    def _get_vehicle(self) -> dict:
        """
        Load vehicle data.

        :return my_vehicle: vehicle data
        :rtype my_vehicle:dict
        """
        session = self._client.session()
        vehicle = session.get(f"{self._client.url}/v1/vehicles").json()
        my_vehicle = dict()
        for v in vehicle["data"]:
            if v["id"] is self.id:
                my_vehicle.add(v)
        return my_vehicle

    # For odometer, battery, charge, location, etc
    def _get(self, item, **kwargs) -> dict:
        """
        Load special vehicle data.

        :param item: e.g odometer, battery, charge, location
        :type item:str
        :param **kwargs: body or path queries
        :type item:str
        :return vehicle_item: information related to item
        :rtype vehicle_item:dict
        """
        data = kwargs.get("data")
        params = kwargs.get("params")

        session = self._client.session()
        vehicle_item = session.get(
            f"{self._client.url}/v1/vehicles/{self.id}/{item}", data=data, params=params
        ).json()
        return vehicle_item

    def get_trips(self, limit=None, start=None, end=None) -> list:
        """
        Load trips of the vehicle.

        :param limit: amount of trips
        :type limit:int or limit:str for "total" amount of trips
        :param start: timestamp from when
        :type start:DateTime
        :param end: timestamp until
        :type end:DateTime
        :return vehicle_trips: list of trips
        :rtype vehicle_trips:list
        """
        params = dict()

        if limit is "total":
            total = self._max("trips")
            params = {"$limit": total}

        if isinstance(limit, int):
            params = {"$limit": limit}
        session = self._client.session()
        vehicle_trips = session.get(
            f"{self._client.url}/v1/vehicles/{self.id}/trips", params=params
        ).json()

        if start is not None or end is not None:

            if isinstance(start, str):
                start = dateutil.parser.parse(start)
            if isinstance(end, str):
                end = dateutil.parser.parse(end)

            if start is None:
                start = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime(0))
                start = dateutil.parser.parse(start)
            if end is None:
                end = datetime.datetime.utcnow()

            ranged_trips = list()

            for trip in vehicle_trips["data"]:
                trip_start = dateutil.parser.parse(trip["startTime"])
                trip_end = dateutil.parser.parse(trip["endTime"])
                if trip_start >= start and trip_end <= end:
                    ranged_trips.append(trip)
            return ranged_trips

        return vehicle_trips["data"]

    # Maybe merge charges and trips in one function
    def get_charges(self, limit=None, start=None, end=None):
        """
        Load charges of the vehicle.

        :param limit: amount of charges
        :type limit:int or limit:str for "total" amount of charges
        :param start: timestamp from when
        :type start:DateTime
        :param end: timestamp until
        :type end:DateTime
        :return vehicle_charges: list of charges
        :rtype vehicle_charges:list
        """

        params = dict()
        if limit is "total":
            params = self._max("charges")

        if limit is int:
            params = {"$limit": limit}
        session = self._client.session()
        vehicle_charges = session.get(
            f"{self._client.url}/v1/vehicles/{self.id}/charges", params=params
        ).json()

        if start is not None or end is not None:

            if isinstance(start, str):
                start = dateutil.parser.parse(start)
            if isinstance(end, str):
                end = dateutil.parser.parse(end)

            if start is None:
                start = time.strftime("%Y-%m-%dT%H:%M:%SZ", time.gmtime(0))
                start = dateutil.parser.parse(start)
            if end is None:
                end = datetime.datetime.utcnow()

            ranged_charges = list()

            for charge in vehicle_charges["data"]:
                charge_start = dateutil.parser.parse(charge["startTime"])
                charge_end = dateutil.parser.parse(charge["endTime"])
                if charge_start >= start and charge_end <= end:
                    ranged_charges.append(charge)
            return ranged_charges

        return vehicle_charges["data"]

    def _max(self, item) -> str:
        data = self._get(item)
        return data.get("total")
